==> atlassian_connect_host-0.5.0.tar.gz: lib/src/host_service/host_service.dart <==
library ac_dart.host_service.service;

import 'package:atlassian_connect_host/host_event_model.dart';
import 'package:atlassian_connect_host/host_registration_persistence.dart';
import 'package:atlassian_connect_host/src/preconditions.dart';
import 'package:atlassian_connect_host/src/http_method.dart';
import 'dart:async';
import 'dart:io';
import 'package:path/path.dart' as _path;
import 'dart:convert';
import 'package:xml/xml.dart';
export 'package:xml/xml.dart';
import 'package:option/option.dart';
import 'package:logging/logging.dart';
import 'package:atlassian_connect_host/src/host_service/host_http_client.dart';
export 'package:atlassian_connect_host/src/host_service/host_http_client.dart'
    show AcHostHttpClient;
import 'package:atlassian_connect_jwt/atlassian_connect_jwt.dart'
    hide createAddonSessionToken;
import 'package:atlassian_connect_jwt/atlassian_connect_jwt.dart'
    as session_token show createAddonSessionToken;
import 'package:either/either.dart';
import 'package:dart_jwt/dart_jwt.dart';

part 'host_service_registration.dart';
part 'host_service_auth.dart';

const Utf8Codec UTF8 = const Utf8Codec();

_path.Context get _url => _path.url;

final Logger _log = new Logger('AcHostService');

typedef AcHostHttpClient HostClientFactory(
    AcHostRegistration hostRegistration, AddonHost addonHost);

AcHostHttpClient _defaultHostClientFactory(
        AcHostRegistration hostRegistration, AddonHost addonHost) =>
    new AcHostHttpClient.fromHostRego(hostRegistration, addonHost);

/**
 * A service for interacting with an Atlassian product host. Acts as a facade
 * for all host related activities
 */
class AcHostService {
  final AcHostRegistrationRepository _hostRegistrationRepository;
  final HostClientFactory _hostClientFactory;
  final AddonHost _addonHost;
  final _AcHostServiceRegistration _hostRegistrationService;
  final _AcHostServiceAuthentication _hostServiceAuthentication;

  /// Note: _defaultHostClientFactory really exposed for testing.
  /// Don't anticipate non test use
  AcHostService(AcHostRegistrationRepository hostRegistrationRepository,
      AddonHost addonHost,
      [this._hostClientFactory = _defaultHostClientFactory])
      : this._hostRegistrationRepository = checkNotNull(
          hostRegistrationRepository),
        this._addonHost = checkNotNull(addonHost),
        this._hostRegistrationService = new _AcHostServiceRegistration(
            addonHost),
        this._hostServiceAuthentication = new _AcHostServiceAuthentication(
            hostRegistrationRepository, addonHost);

  // TODO: May be best to separate event handling services from core services. e.g.
  // AcHostLifecycleService.handleInstallationEvent(InstallationEvent installationEvent)
  // AcHostService.registerHost(AcHostRegistration hostRegistration)
  /**
   * Handles the installation event by registering the Atlassian product host
   */
  Future<AcHostRegistration> registerHost(InstallationEvent installationEvent) {
    final hostRegistration = installationEvent.hostRegistration;

    /*
     * TODO(anders):
     * 1/ validate registration object against schema?
     * 2/ check public key matches that on host => done
     * 3/ make sure host not already registered - prevent hijacking - done
     */
    return _hostRegistrationService
        .verifyRegistration(hostRegistration, httpClient(hostRegistration))
        .then(
            (_) => findRegistrationByHostKey(hostRegistration.hostDetails.key))
        .then((Option<AcHostRegistration> existingRego) {
      if (existingRego is Some) {
        throw new ArgumentError("Host registration exists already for "
            "${hostRegistration.hostDetails.key}");
      } else {
        return _hostRegistrationRepository.create(hostRegistration);
      }
    }).then((savedRegistration) {
      _log.info(
          'Successfully registered with host ${savedRegistration.hostDetails}');
      _log.finest("shared secret is '${savedRegistration.sharedSecret}'");
      return new Future.value(savedRegistration);
    });
  }

  /**
   * Initiates an installation of the addon into the host with the given base
   * uri. Note this is typically only done in development as in production
   * addons are installed via the marketplace.
   */
  Future installAddonToHost(Uri hostBaseUri) {
    return _hostRegistrationService.installAddonToHost(
        hostBaseUri, new AcHostHttpClient(hostBaseUri));
  }

  /**
   * Creates an [AcHostHttpClient] for communicating with the host specified
   * by the [hostRegistration]
   */
  AcHostHttpClient httpClient(AcHostRegistration hostRegistration) =>
      _hostClientFactory(hostRegistration, _addonHost);

  /**
   * Creates an [AcHostHttpClient] for communicating with the host specified
   * by the [hostKey]
   */
  Future<AcHostHttpClient> httpClientForHostKey(String hostKey) {
    return findRegistrationByHostKey(hostKey)
        .then((Option<AcHostRegistration> opt) {
      return opt.map((AcHostRegistration hostRegistration) {
        return httpClient(hostRegistration);
      }).getOrElse(() => new Future.error("unknown host key $hostKey"));
    });
  }

  /**
   * Looks up a [AcHostRegistration] for the given [hostKey]
   */
  Future<Option<AcHostRegistration>> findRegistrationByHostKey(
          String hostKey) =>
      _hostRegistrationRepository.findRegistrationByHostKey(hostKey);

  /**
   * Authenticates a request from an Atlassian host, returning [None] on success
   * and an [AuthenticationFailure] on failure
   */
  Future<Either<AuthenticationFailure, AcSessionContext>> authenticateHostRequest(
      String jwtToken, Uri uri, String method) {
    return _hostServiceAuthentication.authenticateHostRequest(
        jwtToken, uri, method);
  }

  /**
   * Creates a session token for use in addon client and server communications
   */
  String createAddonSessionToken(String hostKey, String user) {
    return _hostServiceAuthentication.createAddonSessionToken(hostKey, user);
  }

  /**
   * Authenticates a session token for use in addon client and server 
   * communications
   */
  Future<Either<AuthenticationFailure, AcSessionContext>> authenticateAddonSessionToken(
      String jwtToken) {
    return _hostServiceAuthentication.authenticateAddonSessionToken(jwtToken);
  }
}

==> atlassian_connect_host-0.5.4.tar.gz: lib/src/host_service/host_service.dart <==
library ac_dart.host_service.service;

import 'package:atlassian_connect_host/host_event_model.dart';
import 'package:atlassian_connect_host/host_registration_persistence.dart';
import 'package:atlassian_connect_host/src/preconditions.dart';
import 'package:atlassian_connect_host/src/http_method.dart';
import 'dart:async';
import 'dart:io';
import 'package:path/path.dart' as _path;
import 'dart:convert';
import 'package:xml/xml.dart';
export 'package:xml/xml.dart';
import 'package:option/option.dart';
import 'package:logging/logging.dart';
import 'package:atlassian_connect_host/src/host_service/host_http_client.dart';
export 'package:atlassian_connect_host/src/host_service/host_http_client.dart'
    show AcHostHttpClient;
import 'package:atlassian_connect_jwt/atlassian_connect_jwt.dart'
    hide createAddonSessionToken;
import 'package:atlassian_connect_jwt/atlassian_connect_jwt.dart'
    as session_token show createAddonSessionToken;
import 'package:either/either.dart';
import 'package:dart_jwt/dart_jwt.dart';
import 'package:shelf_auth/shelf_auth.dart' show JwtSessionHandler, Principal;
import 'package:http_exception/http_exception.dart';

part 'host_service_registration.dart';
part 'host_service_auth.dart';

const Utf8Codec UTF8 = const Utf8Codec();

_path.Context get _url => _path.url;

final Logger _log = new Logger('AcHostService');

typedef AcHostHttpClient HostClientFactory(
    AcHostRegistration hostRegistration, AddonHost addonHost);

AcHostHttpClient _defaultHostClientFactory(
        AcHostRegistration hostRegistration, AddonHost addonHost) =>
    new AcHostHttpClient.fromHostRego(hostRegistration, addonHost);

/**
 * A service for interacting with an Atlassian product host. Acts as a facade
 * for all host related activities
 */
class AcHostService {
  final AcHostRegistrationRepository _hostRegistrationRepository;
  final HostClientFactory _hostClientFactory;
  final AddonHost _addonHost;
  final _AcHostServiceRegistration _hostRegistrationService;
  final _AcHostServiceAuthentication _hostServiceAuthentication;

  /// Note: _defaultHostClientFactory really exposed for testing.
  /// Don't anticipate non test use
  AcHostService(AcHostRegistrationRepository hostRegistrationRepository,
      AddonHost addonHost,
      [this._hostClientFactory = _defaultHostClientFactory])
      : this._hostRegistrationRepository =
            checkNotNull(hostRegistrationRepository),
        this._addonHost = checkNotNull(addonHost),
        this._hostRegistrationService =
            new _AcHostServiceRegistration(addonHost),
        this._hostServiceAuthentication = new _AcHostServiceAuthentication(
            hostRegistrationRepository, addonHost);

  // TODO: May be best to separate event handling services from core services. e.g.
  // AcHostLifecycleService.handleInstallationEvent(InstallationEvent installationEvent)
  // AcHostService.registerHost(AcHostRegistration hostRegistration)
  /**
   * Handles the installation event by registering the Atlassian product host
   */
  Future<AcHostRegistration> registerHost(
      InstallationEvent installationEvent) async {
    final hostRegistration = installationEvent.hostRegistration;

    /*
     * TODO(anders):
     * 1/ validate registration object against schema?
     * 2/ check public key matches that on host => done
     * 3/ make sure host not already registered - prevent hijacking - done
     */
    await _hostRegistrationService.verifyRegistration(
        hostRegistration, httpClient(hostRegistration));

    Option<AcHostRegistration> existingRegistration =
        await findRegistrationByHostKey(hostRegistration.hostDetails.key);

    if (existingRegistration is Some) {
      throw new ArgumentError("Host registration exists already for "
          "${hostRegistration.hostDetails.key}");
    } else {
      final savedRegistration =
          await _hostRegistrationRepository.create(hostRegistration);
      _log.info('Successfully registered with host ${savedRegistration
                .hostDetails}');
      _log.finest("shared secret is '${savedRegistration.sharedSecret}'");
      return savedRegistration;
    }
  }

  /**
   * Initiates an installation of the addon into the host with the given base
   * uri. Note this is typically only done in development as in production
   * addons are installed via the marketplace.
   */
  Future installAddonToHost(Uri hostBaseUri) {
    return _hostRegistrationService.installAddonToHost(
        hostBaseUri, new AcHostHttpClient(hostBaseUri));
  }

  /**
   * Creates an [AcHostHttpClient] for communicating with the host specified
   * by the [hostRegistration]
   */
  AcHostHttpClient httpClient(AcHostRegistration hostRegistration) =>
      _hostClientFactory(hostRegistration, _addonHost);

  /**
   * Creates an [AcHostHttpClient] for communicating with the host specified
   * by the [hostKey]
   */
  Future<AcHostHttpClient> httpClientForHostKey(String hostKey) {
    return findRegistrationByHostKey(hostKey)
        .then((Option<AcHostRegistration> opt) {
      return opt.map((AcHostRegistration hostRegistration) {
        return httpClient(hostRegistration);
      }).getOrElse(() => new Future.error("unknown host key $hostKey"));
    });
  }

  /**
   * Looks up a [AcHostRegistration] for the given [hostKey]
   */
  Future<Option<AcHostRegistration>> findRegistrationByHostKey(
          String hostKey) =>
      _hostRegistrationRepository.findRegistrationByHostKey(hostKey);

  /**
   * Authenticates a request from an Atlassian host, returning [None] on success
   * and an [AuthenticationFailure] on failure
   */
  Future<Either<AuthenticationFailure, AcPrincipal>> authenticateHostRequest(
      String jwtToken, Uri uri, String method) {
    return _hostServiceAuthentication.authenticateHostRequest(
        jwtToken, uri, method);
  }

  /**
   * Creates a session token for use in addon client and server communications
   */
  @deprecated
  String createAddonSessionToken(String hostKey, String user) {
    return _hostServiceAuthentication.createAddonSessionToken(hostKey, user);
  }

  /**
   * Authenticates a session token for use in addon client and server 
   * communications
   */
  @deprecated
  Future<
      Either<AuthenticationFailure, AcPrincipal>> authenticateAddonSessionToken(
      String jwtToken) {
    return _hostServiceAuthentication.authenticateAddonSessionToken(jwtToken);
  }

  JwtSessionHandler<AcPrincipal, AddonSessionClaimSet> get jwtSessionHandler =>
      new AddonSessionAuthenticationHelper(
          _hostRegistrationRepository, _addonHost).jwtSessionHandler;
}
